﻿#!/usr/bin/perl

# Perl program to create Suunto .SDE  package from a simple CSV file (semi-colon ";" separation)
# Created by Maximilien Knopp

# Beta version to manage all char encoding instead of UTF-8 only

#*****************************************
# Declaration of the perl packages needed.
#*****************************************

use strict;
#use warnings;
use Getopt::Long;
use Encode;
require Encode::Detect;
use Archive::Zip qw( :ERROR_CODES :CONSTANTS );

#**********************************
# Declaration of the functions
#**********************************

# *********************************
# Sub getdate
# Return date string for sde format
# It analyse date at the formats: dd/mm/yyyy, dd-mm-yyyy, dd.mm.yyyy, yyyy/mm/dd, yyyy-mm-dd, yyyy.mm.dd
sub getdate
{
	# Variables declaration
  my $input = shift;
  
  if ($input =~ m!^((?:19|20)\d\d)[- /.]([0-9]|0[1-9]|1[012])[- /.]([0-9]|0[1-9]|[12][0-9]|3[01])$!) {
    # At this point, $1 holds the year, $2 the month and $3 the day of the date entered
    if ($3 == 31 and ($2 == 4 or $2 == 6 or $2 == 9 or $2 == 11)) {
      return ""; # 31st of a month with 30 days
    } elsif ($3 >= 30 and $2 == 2) {
      return ""; # February 30th or 31st
    } elsif ($2 == 2 and $3 == 29 and not ($1 % 4 == 0 and ($1 % 100 != 0 or $1 % 400 == 0))) {
      return ""; # February 29th outside a leap year
    } else {
      return sprintf("%02d.%02d.%04d", $3,$2,$1); # Valid date
    }
  }
  elsif ($input =~ m!^([0-9]|0[1-9]|[12][0-9]|3[01])[- /.]([0-9]|0[1-9]|1[012])[- /.]((19|20)\d\d)$!) {
    # At this point, $1 holds the year, $2 the month and $3 the day of the date entered
    if ($1 == 31 and ($2 == 4 or $2 == 6 or $2 == 9 or $2 == 11)) {
      return ""; # 31st of a month with 30 days
    } elsif ($1 >= 30 and $2 == 2) {
      return ""; # February 30th or 31st
    } elsif ($2 == 2 and $3 == 29 and not ($3 % 4 == 0 and ($3 % 100 != 0 or $3 % 400 == 0))) {
      return ""; # February 29th outside a leap year
    } else {
      return sprintf("%02d.%02d.%04d", $1,$2,$3); # Valid date
  	}
  }
  else {
    return ""; # Not a date
  }
}

# *********************************
# Sub gettime
# Return time string for sde format in hh:mm:ss
# It analyse time at the formats: 10:45, 10:45:23, 10h45, 10H45, 10:45 PM/AM, 10h45 am/pm, etc.
sub gettime
{
	# Variables declaration
	my $input = shift;
	my $hour;
	my $minute;
	my $ampm;
	
	if ( $input =~ m!^([0-9]|[01][0-9]|2[0-3])\s*[-.:hH]\s*([0-9]|[0-5][0-9])$! ) {
		return sprintf("%02d:%02d:00",$1,$2);
	}	elsif ( $input =~ m!^([0-9]|[0][0-9]|1[0-2])\s*[-.:hH]\s*([0-9]|[0-5][0-9])\s*([AP-ap]M|[AP-ap]m)$! ) {
		$hour = $1;
		$ampm = $3;
		$ampm =~ tr/a-z/A-Z/;
		if ( $ampm eq "PM" and $hour >= 1 and $hour < 12 )
		{
			$hour += 12;
		}
		elsif ( $ampm eq "AM" and $hour == 12 )
		{
			$hour = "00";
		}
		return sprintf("%02d:%02d:00",$hour,$2);
	} elsif( $input =~ m!^([0-9]|[01][0-9]|2[0-3])\s*[-.:hH]\s*([0-9]|[0-5][0-9])\s*[-.:mM]\s*([0-9]|[0-5][0-9])$! ) {
		return sprintf("%02d:%02d:%02d",$1,$2,$3)
	} elsif( $input =~ m!^([0-9]|[01][0-9]|2[0-3])\s*[-.:hH]\s*([0-9]|[0-5][0-9])! ) {
		return sprintf("%02d:%02d:00",$1,$2);
	}
	else {
		return "";
	}
}


# *********************************
# Sub getduration
# Return 3 variable: 1st the day, 2nd the month, 3rd the year
# It analyse date at the formats: dd/mm/yyyy, dd-mm-yyyy, dd.mm.yyyy, yyyy/mm/dd, yyyy-mm-dd, yyyy.mm.dd
sub getduration
{
	# Variables declaration
	my $input = shift;
	my $duration;
	
	if ( $input =~ m!^([0-9]|[01][0-9]|2[0-3])\s*[-.:hH]\s*([0-9]|[0-5][0-9])\s*[-.:mM]\s*([0-9]|[0-5][0-9])$! ) {
		$duration = $1 * 3600 + $2 * 60 + $3;
		return $duration;
	}	elsif ( $input =~ m!^([0-5][0-9])\s*[-.:mM]\s*([0-5][0-9])$! ) {
		$duration = $1 * 60 + $2;
		return $duration;
	} elsif( $input =~ m!^([0-5][0-9])$! ) {
		$duration = $1 *60;
		return $duration;
	} else {
		return "";
	}
}


#**********************************
# Main 
#**********************************

# Generating type of data to import
# This hash table will associate the type of data with the column number of the CSV file
my %data = (	"date"							=> "",
							"time"							=> "",
							"dive number"				=> "",
							"depth"							=> "",
							"dive duration"			=> "",
							"diver name"				=> "",
							"country"						=> "",
							"location"					=> "",
							"site"							=> "",
							"weather"						=> "",
							"water visibility"	=> "",
							"air temp"					=> "",
							"water temp"				=> "",
							"buddie"						=> "",
							"divemaster"				=> "",
							"boat name"					=> "",
							"presure start"			=> "",
							"presure end"				=> "",
							"gas mode"					=> 0,
							"o2 percent"				=> "",
							"lognote"						=> "", );



# Variables declaration
my $input;
my $analysefirst = 0;
my @dives;
my @dive;
my $cptoutfile = 0;
my $outfile;
my $date;
my $heure;
my $duree;
my $pstart;
my $pend;

# Creation of the sde package (zip file which will be rename in .SDE
my $sdefile = Archive::Zip->new();

# Check the data passed in argument
my ( $filein, $fileout ) = ();
GetOptions(
		'input|i=s' => \$filein,
		'output|o=s' => \$fileout,
		);

# if no input file has been given in argument
if ( ! defined $filein )
{
  print "Usage: perl csv_to_sde.pl -i|-input file.csv [-o|-output file.sde]\n";
  exit 0;
}

# if no output file has been given, set it one by default
if ( ! defined $fileout ) {
  $fileout = "dive_log.sde";
}

# if the input file does not exist
if ( ! -e $filein )
{
	print "The csv file given in input ($filein) does not exist!\n";
	exit 0;
}

# Open the CSV file with UTF-8 encoding
open (FILE, "<",$filein) || die "Ouverture impossible du fichier $ARGV[0]: $!";
@dives = <FILE>;
close (FILE);

print "------------------------------------------------------\n";
print "Enter the COLUMN NUMBER of the data corresponding at :\n";

foreach my $k (keys(%data))
{
	print "-------\n";
	print "$k : ";
	$input = <STDIN>;
	chomp $input;
	if ( $input ne "")
	{
		$data{$k} = $input;
	}
}

print "\n-------\n";
print "The first line has to be analyzed (Y/N) [Y] :";
$input = <STDIN>;
chomp $input;
if ($input =~ m/^[nN]$/)
{
  $analysefirst = 1;
}
print "\n";

for (my $i=0; $i<scalar(@dives); $i++)
{
	# skip first line of the CSV file if needed
	if ( $analysefirst == 1 && $i == 0)
	{
		$i = 1;
	}
	
	# Creation of the xml file for the current dive
	$outfile = $cptoutfile.".xml";
	open(my $out,'>:encoding(latin9)',$outfile);
	
	# Writing header
	print $out "<?xml version=\"1.0\" encoding=\"ISO-8859-15\" ?>\n";
	print $out "<SUUNTO><HEADER><MSGNAME>SDM001A</MSGNAME>\n";
	print $out "<MSGPACKING>1</MSGPACKING>\n";
	print $out "</HEADER>\n";
	print $out "<MSG><WRISTOPID>0</WRISTOPID>\n";
	print $out "<SAMPLECNT>0</SAMPLECNT>\n";
	
	# Extracting data form the current line of the CSV separated with semi-colon (;)
	$dives[$i] = encode("iso-8859-15", decode("Detect", $dives[$i]));
	chomp($dives[$i]);
	$dives[$i] =~ s/"//g;
	@dive = split(/;/,$dives[$i]);
	unshift(@dive,"");
	
	# Writing dive data
	$date = getdate($dive[$data{"date"} // 0]);
	print $out "<DATE>$date</DATE>\n";
	
	$heure = gettime($dive[$data{"time"} // 0]);
	print $out "<TIME>$heure</TIME>\n";
	print $out "<MAXDEPTH>".$dive[$data{"depth"} // 0]."</MAXDEPTH>\n";
	print $out "<MEANDEPTH></MEANDEPTH>\n";
	print $out "<SAMPLEINTERVAL>60</SAMPLEINTERVAL>\n";
	print $out "<LOGTITLE></LOGTITLE>\n";
	print $out "<LOGNOTES>".$dive[$data{"lognote"} // 0]."</LOGNOTES>\n";
	print $out "<FOLDER></FOLDER>\n";
	print $out "<COUNTRY>".$dive[$data{"country"} // 0]."</COUNTRY>\n";
	print $out "<LOCATION>".$dive[$data{"location"} // 0]."</LOCATION>\n";
	print $out "<SITE>".$dive[$data{"site"} // 0]."</SITE>\n";
	print $out "<WEATHER>".$dive[$data{"weather"} // 0]."</WEATHER>\n";
	print $out "<WATERVISIBILITY>".$dive[$data{"water visibility"} // 0]."</WATERVISIBILITY>\n";
	print $out "<AIRTEMP>".$dive[$data{"air temp"} // 0]."</AIRTEMP>\n";
	print $out "<WATERTEMPMAXDEPTH>".$dive[$data{"water temp"} // 0]."</WATERTEMPMAXDEPTH>\n";
	print $out "<WATERTEMPATEND>".$dive[$data{"water temp"} // 0]."</WATERTEMPATEND>\n";	
	print $out "<PARTNER>".$dive[$data{"buddie"} // 0]."</PARTNER>\n";
	print $out "<DIVEMASTER>".$dive[$data{"divemaster"} // 0]."</DIVEMASTER>\n";
	print $out "<BOATNAME>".$dive[$data{"boat name"} // 0]."</BOATNAME>\n";
	print $out "<CYLINDERDESCRIPTION></CYLINDERDESCRIPTION>\n";
	print $out "<CYLINDERSIZE></CYLINDERSIZE>\n";
	print $out "<CYLINDERUNITS>0</CYLINDERUNITS>\n";
	print $out "<CYLINDERWORKPRESSURE></CYLINDERWORKPRESSURE>\n";
	$pstart = $dive[$data{"presure start"} // 0] * 1000;
	print $out "<CYLINDERSTARTPRESSURE>$pstart</CYLINDERSTARTPRESSURE>\n";
	$pend = $dive[$data{"presure end"} // 0] * 1000;
	print $out "<CYLINDERENDPRESSURE>$pend</CYLINDERENDPRESSURE>\n";
	print $out "<DIVENUMBER>".$dive[$data{"dive number"} // 0]."</DIVENUMBER>\n";
	print $out "<DCDIVENUMBER>1</DCDIVENUMBER>\n";
	print $out "<DIVESERIES>0</DIVESERIES>\n";
	print $out "<DEVICEMODEL></DEVICEMODEL>\n";
	
	$duree = getduration($dive[$data{"dive duration"} // 0]);
	print $out "<DIVETIMESEC>$duree</DIVETIMESEC>\n";
	print $out "<PERSONALINFO>".$dive[$data{"diver name"} // 0]."</PERSONALINFO>\n";
	
	$dive[$data{"gas mode"} // 0] =~ s/\s.*//;
	if ( $dive[$data{"gas mode"} // 0] =~ m/[aA]ir/ )
	{
		print $out "<GASMODE>0</GASMODE>\n";
	}
	else
	{
		print $out "<GASMODE>1</GASMODE>\n";
	}
	print $out "<ALTMODE></ALTMODE>\n";
	print $out "<PMODE></PMODE>\n";
	print $out "<WATERVISIBILITYDESC></WATERVISIBILITYDESC>\n";
	print $out "<SACRATE></SACRATE>\n";
	print $out "<OLFPCT></OLFPCT>\n";
	print $out "<WEIGTH></WEIGTH>\n";
	print $out "<O2PCT>".$dive[$data{"o2 percent"} // 0]."</O2PCT>\n";
	print $out "<O2PCT_2></O2PCT_2>\n";
	print $out "<O2PCT_3></O2PCT_3>\n";
	print $out "<OTUPCT></OTUPCT>\n";
	print $out "<CNSPCT></CNSPCT>\n";
	print $out "<PO2MAX></PO2MAX>\n";
	print $out "<RGBMPCT></RGBMPCT>\n";
	print $out "<HPTRANSMITTER></HPTRANSMITTER>\n";
	print $out "<SURFACETIME></SURFACETIME>\n";
	print $out "<NOPROFILE></NOPROFILE>\n";
	print $out "<PREVTPRESSURE_1></PREVTPRESSURE_1>\n";
	print $out "<PREVTPRESSURE_2></PREVTPRESSURE_2>\n";
	print $out "<PREVTPRESSURE_3></PREVTPRESSURE_3>\n";
	print $out "<PREVTPRESSURE_4></PREVTPRESSURE_4>\n";
	print $out "<PREVTPRESSURE_5></PREVTPRESSURE_5>\n";
	print $out "<PREVTPRESSURE_6></PREVTPRESSURE_6>\n";
	print $out "<PREVTPRESSURE_7></PREVTPRESSURE_7>\n";
	print $out "<PREVTPRESSURE_8></PREVTPRESSURE_8>\n";
	print $out "<PREVTPRESSURE_9></PREVTPRESSURE_9>\n";
	print $out "<CUSTOM1></CUSTOM1>\n";
	print $out "<CUSTOM2></CUSTOM2>\n";
	print $out "<CUSTOM3></CUSTOM3>\n";
	print $out "<CUSTOM4></CUSTOM4>\n";
	print $out "<CUSTOM5></CUSTOM5>\n";
	print $out "<SAMPLE><SAMPLETIME>0</SAMPLETIME>\n";
	print $out "<DEPTH></DEPTH>\n";
	print $out "<PRESSURE></PRESSURE>\n";
	print $out "<TEMPERATURE>".$dive[$data{"water temp"} // 0]."</TEMPERATURE>\n";
	print $out "<BOOKMARK></BOOKMARK>\n";
	print $out "<SACRATE></SACRATE>\n";
	print $out "<CYLPRESS></CYLPRESS>\n";
	print $out "</SAMPLE>\n";
	print $out "<GASCHANGE><MIXTIME></MIXTIME>\n";
	print $out "<MIXVALUE></MIXVALUE>\n";
	print $out "<MIXDEPTH></MIXDEPTH>\n";
	print $out "</GASCHANGE>\n";
	print $out "</MSG>\n";
	print $out "</SUUNTO>\n";

	# Closing the xml file for the current dive
	close ($out);
	
	# Adding new file dive to the suunto package
	$sdefile->addFile($outfile);
	
	# Cleaning for next dive
	$cptoutfile++;
}

# Creation of the Suunto dive log file
if ($sdefile->writeToFileNamed($fileout) != AZ_OK)
{
	print "Error while creating Suunto dive log file\n";
}
else
{
	print "Suunto dive log file successfully created\n";
}
